using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using FitConnect;
using FitConnect.Models;
using FluentAssertions;
using Newtonsoft.Json;
using NUnit.Framework;

namespace IntegrationTests.Sender;

[TestFixture]
public class SenderTestHappyPath : SenderTestBase {
    [Test]
    public void CheckIfSecretsAreValid() {
        ClientId.Should().NotBe("00000000-0000-0000-0000-000000000000");
        ClientSecret.Should().NotBe("0000000000000000000000000000000000000000000");
    }



    [TestCase("0b8e6fbd-62e2-4b6f-b333-5308d82e0a00")]
    [TestCase("d2be2027-9368-4c0c-a265-2fdbf7ecd4d9")]
    public void GetSubmissionStatus(string caseId) {
        var status =
            Sender.GetStatusForSubmissionAsync(caseId, "aa3704d6-8bd7-4d40-a8af-501851f93934", true).Result;
        status.ForEach(s => Console.WriteLine($"{s.EventTime} - {s.EventType}"));
        status.Count.Should().BeGreaterThan(0);
    }

    [Test]
    public void Submit_FinishSubmission_ShouldGetIdFromServer() {
        // Arrange

        var attachments = new List<Attachment>();

        var attachment = new Attachment("Test.pdf", "Just an attachment");
        attachments.Add(attachment);
        var dut = SubmissionBuilder.Create()
            .WithDestination(DestinationId)
            .WithServiceType("ServiceName", "urn:de:fim:leika:leistung:99400048079000")
            .WithAttachments(attachments.ToArray())
            .WithData(@"{""data"":""value""}")
            .Build();

        // Act
        var submission = Sender.SendAsync(dut).Result;

        // Assert
        submission.Should().NotBeNull();
        Console.WriteLine($"Case ID: {submission.CaseId}");
    }

    [Test]
    public void Submit_FinishSubmissionWithData_ShouldGetIdFromServer() {
        // Arrange

        var attachments = new List<Attachment>();

        var content = File.ReadAllBytes("Test.pdf");
        var attachment = new Attachment("Test.pdf", "Just an attachment") {
            Filename = "RandomBytes",
            MimeType = "application/pdf",
            Description = "Just a test",
            Purpose = "attachment"
        };
        attachments.Add(attachment);
        var dut = SubmissionBuilder.Create()
            .WithDestination(DestinationId)
            .WithServiceType("ServiceName", "urn:de:fim:leika:leistung:99400048079000")
            .WithAttachments(attachments)
            .WithData(JsonConvert.SerializeObject(new {
                FirstName = "John",
                LastName = "Doe",
                Age = 42,
                Birthday = DateTime.Today.AddYears(-42)
            }))
            .Build();

        // Act
        var submission = Sender.SendAsync(dut).Result;

        // Assert
        var statusForSubmission = Sender.GetStatusForSubmissionAsync(submission).Result;
        foreach (var securityEventToken in statusForSubmission)
            Console.WriteLine(securityEventToken.Token.Subject);

        statusForSubmission.Should().HaveCountGreaterThan(0);
        Console.WriteLine($"Case ID: {submission.CaseId}");
        Console.WriteLine($"Submission ID: {submission.Id}");
        submission.Should().NotBeNull();
    }

    [Ignore("Destination can not be found")]
    [Test]
    public void Submit_FinishSubmissionWithDataFindDestination_ShouldGetIdFromServer() {
        // Arrange

        var attachments = new List<Attachment>();

        var content = File.ReadAllBytes("Test.pdf");
        var attachment = new Attachment("Test.pdf", "Just an attachment") {
            Filename = "RandomBytes",
            MimeType = "application/pdf",
            Description = "Just a test",
            Purpose = "attachment"
        };
        attachments.Add(attachment);

        var route = Sender.FindDestinationId("99400048079000", ars: "09372126").Result.First();

        var dut = SubmissionBuilder.Create()
            .WithDestination(route.DestinationId)
            .WithServiceType("ServiceName", "urn:de:fim:leika:leistung:99400048079000")
            .WithAttachments(attachments)
            .WithData(JsonConvert.SerializeObject(new {
                FirstName = "John",
                LastName = "Doe",
                Age = 42,
                Birthday = DateTime.Today.AddYears(-42)
            }))
            .Build();

        // Act
        var submission =Sender.SendAsync(dut).Result;

        // Assert
        var statusForSubmission = Sender.GetStatusForSubmissionAsync(submission).Result;
        foreach (var securityEventToken in statusForSubmission)
            Console.WriteLine(securityEventToken.Token.Subject);

        statusForSubmission.Should().HaveCountGreaterThan(0);
        Console.WriteLine($"Case ID: {submission.CaseId}");
        Console.WriteLine($"Submission ID: {submission.Id}");
        submission.Should().NotBeNull();
    }
}
